﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Extensions;
using SolidFire.Core.Helpers;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Exceptions;
using SolidFire.Element.Api;

#endregion

namespace SolidFire.Account.Set
{
    /// <summary>
    /// ModifyAccount is used to modify an existing account. When locking an account, any existing connections from that account are
    /// immediately terminated. When changing CHAP settings, any existing connections continue to be active, and the new CHAP values
    /// are only used on subsequent connection or reconnection. There is special handling for the initiator and target secrets
    /// parameters. When these string are the empty string, they will not be sent to the 
    /// ModifyAccount call. If they are null or a non-empty string the value null or the string contents
    /// will be sent. A value of null means to generate a new password.
    /// </summary>
    
    [Cmdlet(VerbsCommon.Set, "SFAccount", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.High)]
    public class SetSFAccount : SFCmdlet
    {
        #region Private Data
        /// <summary>
        /// Member used to determine to send null or a string to the API for the initiatorSecret param.
        /// </summary>
        private string initiatorSecret = string.Empty;

        /// <summary>
        /// Member used to determine to send null or a string to the API for the targetSecret param.
        /// </summary>
        private string targetSecret = string.Empty;

        /// <summary>
        /// Member used to host a nested Pipeline for calling other CmdLets
        /// </summary>
        private Pipeline pipeline = null;

        #endregion

        #region Parameters
        /// <summary>
        /// Id for the account the user is looking to modify.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "AccountID for the account to modify")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public Int64 AccountID { get; set; }

        /// <summary>
        /// Optional status string, acceptable values are "active" and "locked".
        /// </summary>
        [Parameter(Position = 1, Mandatory = false, ValueFromPipeline = false, ValueFromPipelineByPropertyName = false, HelpMessage = "Status to set the account to.")]
        [ValidateSet("active", "locked", IgnoreCase = true)]
        [ValidateNotNullOrEmpty]
        public string Status { get; set; }

        /// <summary>
        /// Optional parameter for specifying an initiator secret for the account. A value of null means to 
        /// generate a new initiator secret from the cluster for the account.
        /// </summary>
        [Parameter(Position = 2, Mandatory = false, ValueFromPipelineByPropertyName = true, HelpMessage = "CHAP secret to use for the initiator")]
        public string InitiatorSecret
        {
            get { return initiatorSecret; }
            set
            {
                if (value.IsExactlyEmpty())
                {
                    var errorRecord =
                            new ErrorRecord(new ParameterException("InitiatorSecret can not be an empty string. To have it re-generated, use $null otherwise pass in a valid value."),
                                    "ParameterException", ErrorCategory.InvalidOperation, null);
                    SolidFireUtilities.HandleError(errorRecord, this);
                }
                initiatorSecret = value;
            }
        }

        /// <summary>
        /// Optional parameter for specifying a target secret for the account. A value of null means to 
        /// generate a new target secret from the cluster for the account.
        /// </summary>
        [Parameter(Position = 3, Mandatory = false, ValueFromPipelineByPropertyName = true, HelpMessage = "CHAP secret to use for the target")]
        public string TargetSecret
        {
            get { return targetSecret; }
            set
            {
                if (value.IsExactlyEmpty())
                {
                    var errorRecord =
                            new ErrorRecord(new ParameterException("TargetSecret can not be an empty string. To have it re-generated, use $null otherwise pass in a valid value."),
                                    "ParameterException", ErrorCategory.InvalidOperation, null);
                    SolidFireUtilities.HandleError(errorRecord, this);
                }
                targetSecret = value;
            }
        }

        [Parameter(Position = 4, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Change username for this account")]
        [ValidatePattern(SolidFireValidations.UpperAndLowerCaseAndNumeric)]
        [ValidateLength(1, 64)]
        public String Username { get; set; }

        /// <summary>
        /// Attributes to save with the account. The overwrites existing attributes assigned to the account.
        /// </summary>
        [Parameter(Position = 5, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Attributes to set")]
        public Hashtable Attributes { get; set; }

        #endregion

        #region Cmdlet Overrides

        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            CheckConnection();
            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
        }

        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFAccount");
            command.Parameters.Add("AccountID", AccountID);
            if (SFConnection != null)
            {
                command.Parameters.Add("SFConnection", SFConnection);
            }
            else if (Target != null)
            {
                command.Parameters.Add("Target", Target);
            }
            pipeline.Commands.Add(command);
            WriteObject(pipeline.Invoke(), true);
            pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            // Assemble the parameter object
            var request = new ModifyAccountRequest();
            request.AccountID = AccountID;

            if (!string.IsNullOrEmpty(Status))
            {
                request.Status = Status;
            }

            if (!string.IsNullOrEmpty(Username))
            {
                request.Username = Username;
            }


            if (initiatorSecret == null)
            {
                request.InitiatorSecret = CHAPSecret.AutoGenerate();
            }
            else if (!initiatorSecret.IsExactlyEmpty())
            {
                request.InitiatorSecret = new CHAPSecret(initiatorSecret);
            }

            if (targetSecret == null)
            {
                request.TargetSecret = CHAPSecret.AutoGenerate();
            }
            else if (!targetSecret.IsExactlyEmpty())
            {
                request.TargetSecret = new CHAPSecret(targetSecret);
            }

            if (this.Attributes != null)
            {
                request.Attributes = Attributes;
            }
            SendRequest<ModifyAccountResult>(method: "ModifyAccount", request: request);

        }
        #endregion

        #region Methods

        #endregion
    }
}