﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Management.Automation;
using SolidFire.Core.Helpers;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
#endregion

namespace SolidFire.Cluster.Add
{
    /// <summary>
    /// 
    /// </summary>
    
    [Cmdlet(VerbsCommon.Add, "SFSnmpNetwork", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.None)]
    public class AddSFSnmpNetwork : SFCmdlet
    {
        #region Private Data

        /// <summary>
        /// Member used to host a nested Pipeline for calling other CmdLets
        /// </summary>
        private Pipeline pipeline = null;

        #endregion

        #region Parameters
        /// <summary>
        /// A Cidr network mask. This network mask must be an integer greater than or equal to 0, and less than or equal to 32. It must also not be equal to 31.
        /// </summary>
        [Parameter(Position = 1, Mandatory = true, HelpMessage = "A Cidr network mask. This network mask must be an integer greater than or equal to 0, and less than or equal to 32. It must also not be equal to 31.")]
        public int Cidr { get; set; }

        /// <summary>
        /// ro: read-only access.*, rw: for read-write access, rosys: for read-only access to a restricted set of system information.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, HelpMessage = "ro: read-only access.*, rw: for read-write access, rosys: for read-only access to a restricted set of system information.")]
        [ValidatePattern(SolidFireValidations.UpperAndLowerCase)]
        [ValidateSet("ro", "rw", "rosys", IgnoreCase = true)]
        public String Access { get; set; }

        /// <summary>
        /// SNMP community string
        /// </summary>
        [Parameter(Position = 2, Mandatory = true, HelpMessage = "SNMP community string.")]
        public String Community { get; set; }


        /// <summary>
        /// This parameter along with the cidr variable is used to control which network the access and community string apply to. The special value of "default" is used to specify an entry that applies to all networks. The cidr mask is ignored when network value is either a host name or default.
        /// </summary>
        [Parameter(Position = 3, Mandatory = true, HelpMessage = "This parameter along with the cidr variable is used to control which network the access and community string apply to. The special value of 'default' is used to specify an entry that applies to all networks. The cidr mask is ignored when network value is either a host name or default.")]
        public String Network { get; set; }


        #endregion

        #region Cmdlet Overrides

        /// <summary>
        /// Make sure the connection is Oxygen or greater.
        /// Create a nested pipeline that can be written to after processing
        /// </summary>
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            CheckConnection(minVersionNumber: 8.0f);
            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
        }

        /// <summary>
        /// After processing, invoke Get-SFSnmpACL so the latest is passed into the pipeline.
        /// </summary>
        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFSnmpInfo");
            pipeline.Commands.Add(command);
            // Send it to the pipline
            WriteObject(pipeline.Invoke(), true);
            pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            // Get the current TrapRecipients
            var objsFromAPI = SendRequest<GetSnmpInfoResult>("GetSnmpInfo");
            foreach (var obj in objsFromAPI)
            {
                var networks = obj.Result.Networks == null ? new List<SnmpNetwork>() : obj.Result.Networks.ToList();

                var newNetwork = new SnmpNetwork
                {
                    Access = Access,
                    Community = Community,
                    Cidr = Cidr,
                    Network = Network
                };

                networks.Add(newNetwork);
                
                var request = new SetSnmpInfoRequest();
                request.Networks = networks.ToArray();
                request.SnmpV3Enabled = false;
                request.Enabled = true;

                if (WhatIfShouldProcessHandler(obj.Target.Target)) 
                {
                    var objFromAPI = SingleConnectionRequest<SetSnmpInfoResult>("SetSnmpInfo", obj.Target, request);

                    if (SolidFireUtilities.HasAPIError(objFromAPI.Error, this)) return;
                }
            }
        }

        #endregion
    }
}