﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Validation;
using SolidFire.Core;
using SolidFire.Core.Helpers;
using SolidFire.Element.Api;

#endregion

namespace SolidFire.Cluster.New
{
    /// <summary>
    /// The CreateCluster method is used to initialize the node in a cluster that has ownership of the "mvip" and "svip" addresses. Each
    /// new cluster is initialized using the MIP of the first node in the cluster. This method also automatically adds all the nodes being
    /// configured into the cluster. The method is used only once each time a new cluster is initialized.
    /// </summary>
    
    [Cmdlet(VerbsCommon.New, "SFCluster", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.Medium)]
    public class NewSFCluster : SFCmdlet
    {
        #region Private Data

        private string mvip;
        private string svip;
        private int repCount = 2;
        private string username;
        private string password;
        private string[] nodes;
        

        #endregion

        #region Parameters
        /// <summary>
        /// 
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Floating (virtual) IP address for the cluster on the management network")]
        [ValidatePattern(SolidFireValidations.IPAddress)]
        public String Mvip
        {
            get { return mvip; }
            set { mvip = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        [Parameter(Position = 1, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Floating (virtual) IP address for the cluster on the storage (iSCSI) network")]
        [ValidatePattern(SolidFireValidations.IPAddress)]
        public string Svip
        {
            get { return svip; }
            set { svip = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        [Parameter(Position = 2, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "User name for the cluster admin")]
        [ValidatePattern(SolidFireValidations.UpperAndLowerCaseAndNumeric)]
        public string Username
        {
            get { return username; }
            set { username = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        [Parameter(Position = 3, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Initial password for the cluster admin account")]
        public string Password
        {
            get { return password; }
            set { password = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        [Parameter(Position = 4, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "IP addresses of the initial set of nodes making up the cluster. This node's IP must be on the list")]
        public string[] Nodes
        {
            get { return nodes; }
            set { nodes = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        [Parameter(Position = 5, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Number of replicas of each piece of data to store in the cluster")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public int ReplicaCount
        {
            get { return repCount; }
            set { repCount = value; }
        }
        /// <summary>
        /// 
        /// </summary>
        [Parameter(HelpMessage = "Include this switch parameter if you accept the EULA.")]
        public SwitchParameter AcceptEula { get; set; }
        /// <summary>
        /// 
        /// </summary>
        [Parameter(Position = 6, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Dictionary of name/values")]
        public Hashtable Attributes { get; set; }

        #endregion

        #region Cmdlet Overrides

        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            if (!AcceptEula.IsPresent)
            {
                var errDetails = new ErrorDetails("You must accept the EULA by including the -AcceptEula parameter.");
                var errRecord = new ErrorRecord(new ArgumentException(), errDetails.Message, ErrorCategory.SecurityError, this);
                SolidFireUtilities.HandleError(errRecord, this);
            }
            CheckConnection(endPoint: SolidFireUtilities.SFEndPoint.Node);
        }

        /// <summary>
        /// 
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            // Assemble the parameters object
            var request = new CreateClusterRequest();
            request.Attributes = Attributes;
            request.Mvip = mvip;
            request.Nodes = nodes;
            request.Password = password;
            request.Username = username;
            request.RepCount = repCount;
            request.Svip = svip;
            request.AcceptEula = true;

            SendRequest<CreateClusterResult>(method: "CreateCluster", request: request, force443: true);
        }


        #endregion
    }
}