﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
using SolidFire.Core.Helpers;
using SolidFire.Core.Objects;

#endregion

namespace SolidFire.Cluster.New
{
    /// <summary>
    /// NewClusterAdmin adds a new Cluster Admin. A Cluster Admin can be used to manage the cluster via the API and
    /// management tools. Cluster Admins are completely separate and unrelated to standard tenant accounts.
    /// Each Cluster Admin can be restricted to a sub-set of the API. SolidFire recommends using multiple Cluster Admins for different
    /// users and applications. Each Cluster Admin should be given the minimal permissions necessary to reduce the potential
    /// impact of credential compromise.
    /// </summary>

    [Cmdlet(VerbsCommon.New, "SFClusterAdmin", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.None)]
    public class NewSFClusterAdmin : SFCmdlet
    {
        #region Private Data

        /// <summary>
        /// Member used to host a nested Pipeline for calling other CmdLets
        /// </summary>
        private Pipeline pipeline = null;

        #endregion

        #region Parameters

        /// <summary>
        /// Unique username for this Cluster Admin.
        /// </summary>
        [Parameter(Mandatory = true, ValueFromPipeline = true, HelpMessage = "Unique username for this Cluster Admin.")]
        public string Username { get; set; }

        /// <summary>
        /// The password of the user. Must be between 8 and 255 characters long (inclusive). No blanks allowed. Required if "secLevel" is "auth" or "priv."
        /// </summary>
        [Parameter(Mandatory = true, ValueFromPipeline = true, HelpMessage = "Password used to authenticate this Cluster Admin.")]
        public string Password { get; set; }

        /// <summary>
        /// Controls which methods this Cluster Admin can use. For more details on the levels of access, see the Access Control appendix of the SolidFire API.
        /// </summary>
        [Parameter(Mandatory = true, ValueFromPipeline = true, HelpMessage = "Controls which methods this Cluster Admin can use. For more details on the levels of access, see the Access Control appendix of the SolidFire API.")]
        [ValidateSet("read", "nodes", "reporting", "drives", "volumes", "accounts", "administrator")]
        public string[] Access { get; set; }

        /// <summary>
        ///
        /// </summary>
        [Parameter(HelpMessage = "Include this switch parameter if you accept the EULA.")]
        public SwitchParameter AcceptEula { get; set; }

        /// <summary>
        /// List of Name/Value pairs in JSON object format.
        /// </summary>
        [Parameter(Mandatory = false, ValueFromPipeline = false, HelpMessage = "List of Name/Value pairs in JSON object format.")]
        public Hashtable Attributes { get; set; }


        #endregion

        #region Cmdlet Overrides

        /// <summary>
        /// Create a nested pipeline that can be written to after processing
        /// </summary>
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            if (SFConnection != null)
            {
                foreach (var conn in SFConnection)
                {
                    _checkAcceptEula(conn);
                }
            }
            else if (Target != null)
            {
                foreach (var target in Target)
                {
                    var conn = SolidFireUtilities.GetSFConnection(this, target);
                    _checkAcceptEula(conn);
                }
            }

            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
            CheckConnection();
        }

        private void _checkAcceptEula(SFConnection conn)
        {
            if (AcceptEula.IsPresent || !(conn.Element.GetConnectionVersion() >= 8.0f)) return;
            var errDetails =
                new ErrorDetails("You must accept the EULA by including the -AcceptEula parameter.");
            var errRecord = new ErrorRecord(new ArgumentException(), errDetails.Message,
                ErrorCategory.SecurityError, this);
            SolidFireUtilities.HandleError(errRecord, this);
        }

        /// <summary>
        /// After processing, invoke Get-SFClusterAdmin so the latest list of admins is passed into the pipeline
        /// </summary>
        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFClusterAdmin");
            pipeline.Commands.Add(command);
            // Send it to the pipline
            WriteObject(pipeline.Invoke(), true);
            pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            // Assemble the parameters object
            var request = new AddClusterAdminRequest();
            request.Access = Access;
            request.Password = Password;
            request.Username = Username;
            request.AcceptEula = true;
            if (this.Attributes != null) { request.Attributes = Attributes; }

            SendRequest<AddClusterAdminResult>("AddClusterAdmin", request);

        }

        #endregion
    }
}