﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections;
using System.ComponentModel;
using System.Management.Automation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
using SolidFire.Core.Helpers;
using SolidFire.Core.Objects;

#endregion

namespace SolidFire.Cluster.New
{
    /// <summary>
    /// AddLdapClusterAdmin is used to add a new LDAP Cluster Admin. An LDAP Cluster Admin can be used to manage the cluster
    /// via the API and management tools. LDAP Cluster Admins are completely separate and unrelated to standard tenant accounts.
    /// 
    /// An LDAP group that has been defined in Active Directory can also be added using this API method. The access level that is
    /// given to the group will be passed to the individual users in the LDAP group.
    /// </summary>
    
    [Cmdlet(VerbsCommon.New, "SFLdapClusterAdmin")]
    public class NewSFLdapClusterAdmin : SFCmdlet
    {
        #region Private Data

        private Pipeline _pipeline = null;

        #endregion

        #region Parameters

        /// <summary>
        /// The distinguished user name for the new LDAP cluster admin.
        /// </summary>
        [Parameter(Mandatory = true, ValueFromPipeline = true, HelpMessage = "The distinguished user name for the new LDAP cluster admin.")]
        public String Username { get; set; }

        /// <summary>
        /// Controls which methods this Cluster Admin can use. For more details on the levels of access, see the Access Control appendix in the SolidFire API Reference.
        /// </summary>
        [Parameter(Mandatory = true, ValueFromPipeline = true, HelpMessage = "Controls which methods this Cluster Admin can use. For more details on the levels of access, see the Access Control appendix in the SolidFire API Reference.")]
        public String[] Access { get; set; }

        /// <summary>
        /// The password of the user. Must be between 8 and 255 characters long (inclusive). No blanks allowed. Required if "secLevel" is "auth" or "priv."
        /// </summary>
        [Parameter(HelpMessage = "Include this switch parameter if you accept the EULA.")]
        public SwitchParameter AcceptEula { get; set; }

        /// <summary>
        /// List of Name/Value pairs in JSON object format.
        /// </summary>
        [Parameter(Mandatory = false, ValueFromPipeline = false, HelpMessage = "List of Name/Value pairs in JSON object format.")]
        public Hashtable Attributes { get; set; }

        #endregion

        #region Cmdlet Overrides

        /// <summary>
        /// Create a nested pipeline that can be written to after processing
        /// </summary>
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            if (SFConnection != null)
            {
                foreach (var conn in SFConnection)
                {
                    _checkAcceptEula(conn);
                }
            }
            else if (Target != null)
            {
                foreach (var target in Target)
                {
                    var conn = SolidFireUtilities.GetSFConnection(this, target);
                    _checkAcceptEula(conn);
                }
            }

            _pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
            CheckConnection(minVersionNumber: 8.0f);
        }

        private void _checkAcceptEula(SFConnection conn)
        {
            if (AcceptEula.IsPresent || !(conn.Element.GetConnectionVersion() >= 8.0f)) return;
            var errDetails =
                new ErrorDetails("You must accept the EULA by including the -AcceptEula parameter.");
            var errRecord = new ErrorRecord(new ArgumentException(), errDetails.Message,
                ErrorCategory.SecurityError, this);
            SolidFireUtilities.HandleError(errRecord, this);
        }

        /// <summary>
        /// After processing, invoke Get-SFClusterAdmin so the latest list of admins is passed into the pipeline
        /// </summary>
        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFClusterAdmin");
            _pipeline.Commands.Add(command);
            // Send it to the pipline
            WriteObject(_pipeline.Invoke(), true);
            _pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            // Assemble the parameters object
            var request = new AddLdapClusterAdminRequest();
            request.Access = Access;
            request.Username = Username;
            request.AcceptEula = true;
            request.Attributes = Attributes;

            SendRequest<AddLdapClusterAdminResult>("AddLdapClusterAdmin", request);

        }

        #endregion
    }
}