﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
using System.Linq;
#endregion

namespace SolidFire.Cluster.New
{
    /// <summary>
    /// AddVirtualNetwork is used to add a new virtual network to a cluster configuration. When a virtual network is added, an
    //  interface for each node is created and each will require a virtual network IP address. The number of IP addresses specified as
    //  a parameter for this API method must be equal to or greater than the number of nodes in the cluster. Virtual network addresses
    //  are bulk provisioned by SolidFire and assigned to individual nodes automatically. Virtual network addresses do not need to be
    //  assigned to nodes manually.
    /// </summary>
    
    [Cmdlet(VerbsCommon.New, "SFVirtualNetwork", SupportsShouldProcess = true)]
    public class NewSFVirtualNetwork : SFCmdlet
    {
        #region Private Data

        /// <summary>
        /// Member used to host a nested Pipeline for calling other CmdLets
        /// </summary>
        private Pipeline pipeline = null;

        #endregion

        #region Parameters

        /// <summary>
        /// A unique virtual network (VLAN) tag. Supported values are 1 to 4095 (the number zero (0) is not supported).
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline=true, HelpMessage = "A unique virtual network (VLAN) tag. Supported values are 1 to 4095 (the number zero (0) is not supported).")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        [ValidateRange(1,4095)]
        public Int64 VirtualNetworkTag { get; set; }

        /// <summary>
        /// User defined name for the new virtual network.
        /// </summary>
        [Parameter(Position = 1, Mandatory = true, ValueFromPipeline = true, HelpMessage = "User defined name for the new virtual network.")]
        public String Name { get; set; }

        /// <summary>
        /// Unique Range of IP addresses to include in the virtual network. 
        ///  Attributes for this parameter are:
        ///  start: start of the IP address range. (String)
        ///  size: number of IP addresses to include in the block. (Integer)
        /// </summary>
        [Parameter(Position = 2, Mandatory = true, ValueFromPipeline = true, HelpMessage = "Unique Range of IP addresses to include in the virtual network. List of AddressBlock objects with 'start' and 'size' for the starting IP address range and size of addresses in block.")]
        public AddressBlock[] AddressBlocks { get; set; }

        /// <summary>
        /// User defined netmask for the new virtual network.
        /// </summary>
        [Parameter(Position = 3, Mandatory = true, ValueFromPipeline = true, HelpMessage = "Unique netmask for the virtual network being created.")]
        public String Netmask { get; set; }

        /// <summary>
        /// Unique storage IP address for the virtual network being created.
        /// </summary>
        [Parameter(Position = 4, Mandatory = true, ValueFromPipeline = true, HelpMessage = "Unique storage IP address for the virtual network being created.")]
        public String Svip { get; set; }

        /// <summary>
        /// List of Name/Value pairs in JSON object format.
        /// </summary>
        [Parameter(Mandatory = false, Position = 5, ValueFromPipeline = false, HelpMessage = "List of Name/Value pairs in JSON object format.")]
        public Hashtable Attributes { get; set; }


        #endregion

        #region Cmdlet Overrides

        /// <summary>
        /// Create a nested pipeline that can be written to after processing
        /// </summary>
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
            CheckConnection(minVersionNumber: 7.0f);
        }

        /// <summary>
        /// After processing, invoke Get-SFClusterAdmin so the latest list of admins is passed into the pipeline
        /// </summary>
        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFVirtualNetwork");
            command.Parameters.Add("VirtualNetworkTag", VirtualNetworkTag);
            if (SFConnection != null)
            {
                command.Parameters.Add("SFConnection", SFConnection);
            }
            else if (Target != null)
            {
                command.Parameters.Add("Target", Target);
            }
            pipeline.Commands.Add(command);
            
            // Send it to the pipline
            WriteObject(pipeline.Invoke(), true);
            pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            // Assemble the parameters object
            var request = new AddVirtualNetworkRequest();
            request.AddressBlocks = AddressBlocks.Select(a => new AddressBlockParams {Size = a.Size, Start = a.Start}).ToArray();
            request.Name = Name;
            request.Netmask = Netmask;
            request.Svip = Svip;
            request.VirtualNetworkTag = VirtualNetworkTag;

            if (Attributes != null) { request.Attributes = Attributes; }
            
           SendRequest<AddVirtualNetworkResult>("AddVirtualNetwork", request);

        }

        #endregion
    }
}
