﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Management.Automation;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
#endregion

namespace SolidFire.Volume.Set
{
    /// <summary>
    /// RemoveClusterAdmin is used to remove a Cluster Admin. The "admin" Cluster Admin cannot be removed.
    /// </summary>

    [Cmdlet(VerbsCommon.Remove, "SFClusterAdmin", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.High)]
    public class RemoveSFClusterAdmin : SFCmdlet
    {
        #region Private Data

        /// <summary>
        /// Member used to host a nested Pipeline for calling other CmdLets
        /// </summary>
        private Pipeline pipeline = null;

        #endregion

        #region Parameters

        /// <summary>
        /// Array of ClusterAdminIDs for the Cluster Admin to remove.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ParameterSetName = "ClusterAdminID", ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "ClusterAdminID for the Cluster Admin to remove.")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public Int64[] ClusterAdminID { get; set; }

        /// <summary>
        /// Array of ClusterAdmin Usernames to be removed.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ParameterSetName = "ClusterAdminName", ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Name the Cluster Admin to remove. Must be exact match. Case sensitive.")]
        public String[] Username { get; set; }

        /// <summary>
        /// Array of ClusterAdmins to be removed
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ParameterSetName = "ClusterAdmin", ValueFromPipeline = true, HelpMessage = "ClusterAdmin for the Cluster Admin to remove.")]
        public ClusterAdmin[] ClusterAdmin { get; set; }

        #endregion

        #region Cmdlet Overrides

        /// <summary>
        /// Create a nested pipeline that can be written to after processing
        /// </summary>
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
            CheckConnection();
        }

        /// <summary>
        /// After processing, invoke the Get-SFClusterAdmin call and write the updated list of Cluster Admins to the pipeline.
        /// </summary>
        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFClusterAdmin");
            pipeline.Commands.Add(command);
            // Send it to the pipline
            WriteObject(pipeline.Invoke(), true);
            pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            List<Int64> idsToRemove = new List<Int64>();
            switch (ParameterSetName)
            {
                case "ClusterAdminID":
                    foreach (var id in ClusterAdminID)
                    {
                        idsToRemove.Add(id);
                    }
                    removeByID(idsToRemove);
                    break;
                case "ClusterAdminName":
                    removeByName(Username.ToList());
                    break;
                case "ClusterAdmin":
                    foreach (var admin in ClusterAdmin)
                    {
                        idsToRemove.Add(admin.ClusterAdminID);
                    }
                    removeByID(idsToRemove);
                    break;
            }
        }

        private void removeByID(List<Int64> idsToRemove)
        {
            foreach (var id in idsToRemove)
            {
                // Assemble the parameters object
                var request = new RemoveClusterAdminRequest();
                request.ClusterAdminID = id;

                SendRequest<RemoveClusterAdminResult>("RemoveClusterAdmin", request);
            }
        }

        private void removeByName(List<string> namesToRemove)
        {
            var objsFromAPI = SendRequest<ListClusterAdminsResult>("ListClusterAdmins");
            foreach (var obj in objsFromAPI)
            {
                var matchedAdmins = obj.Result.ClusterAdmins.Where(a => namesToRemove.Contains(a.Username));
                foreach (var admin in matchedAdmins)
                {
                    // Assemble the parameters object
                    var request = new RemoveClusterAdminRequest();
                    request.ClusterAdminID = admin.ClusterAdminID;

                    var returnable = SingleConnectionRequest<RemoveClusterAdminResult>("RemoveClusterAdmin", obj.Target, request).Result;
                }
            }
        }
        #endregion
    }
}