﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections;
using System.ComponentModel;
using System.Linq;
using System.Management.Automation;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
#endregion

namespace SolidFire.Cluster.New
{
    /// <summary>
    /// NewClusterAdmin adds a new Cluster Admin. A Cluster Admin can be used to manage the cluster via the API and
    /// management tools. Cluster Admins are completely separate and unrelated to standard tenant accounts.
    /// Each Cluster Admin can be restricted to a sub-set of the API. SolidFire recommends using multiple Cluster Admins for different
    /// users and applications. Each Cluster Admin should be given the minimal permissions necessary to reduce the potential
    /// impact of credential compromise.
    /// </summary>
    
    [Cmdlet(VerbsCommon.Set, "SFClusterAdmin", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.High)]
    public class SetSFClusterAdmin : SFCmdlet
    {
        #region Private Data

        /// <summary>
        /// Member used to host a nested Pipeline for calling other CmdLets
        /// </summary>
        private Pipeline pipeline = null;

        #endregion

        #region Parameters

        /// <summary>
        /// ClusterAdminID for the Cluster Admin or LDAP Cluster Admin to modify.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, HelpMessage = "ClusterAdminID for the Cluster Admin or LDAP Cluster Admin to modify.")]
        public Int64 ClusterAdminID { get; set; }
        
        /// <summary>
        /// The password of the user. Must be between 8 and 255 characters long (inclusive). No blanks allowed. Required if "secLevel" is "auth" or "priv."
        /// </summary>
        [Parameter(Position = 1, Mandatory = false, ValueFromPipeline = true, HelpMessage = "Password used to authenticate this Cluster Admin.")]
        public String Password { get; set; }


        /// <summary>
        /// Controls which methods this Cluster Admin can use. For more details on the levels of access, see the Access Control appendix of the SolidFire API.
        /// </summary>
        [Parameter(Position = 2, Mandatory = false, ValueFromPipeline = true, HelpMessage = "Controls which methods this Cluster Admin can use. For more details on the levels of access, see the Access Control appendix of the SolidFire API.")]
        [ValidatePattern(SolidFireValidations.UpperAndLowerCase)]
        public String[] Access { get; set; }

        /// <summary>
        /// List of Name/Value pairs in JSON object format.
        /// </summary>
        [Parameter(Mandatory = false, Position = 3, ValueFromPipeline = false, HelpMessage = "List of Name/Value pairs in JSON object format.")]
        public Hashtable Attributes { get; set; }


        #endregion

        #region Cmdlet Overrides

        /// <summary>
        /// Create a nested pipeline that can be written to after processing
        /// </summary>
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
            CheckConnection();
        }

        /// <summary>
        /// After processing, invoke Get-SFClusterAdmin so the latest list of admins is passed into the pipeline
        /// </summary>
        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFClusterAdmin");
            pipeline.Commands.Add(command);
            // Send it to the pipline
            WriteObject(pipeline.Invoke().Where(p => ((ClusterAdmin)p.BaseObject).ClusterAdminID == ClusterAdminID ));
            pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            // Assemble the parameters object
            var request = new ModifyClusterAdminRequest();
            request.ClusterAdminID = ClusterAdminID;
            if (Access != null) { request.Access = Access; }
            if (Password != null) { request.Password = Password; }
            if (this.Attributes != null) { request.Attributes = Attributes; }

            SendRequest<ModifyClusterAdminResult>("ModifyClusterAdmin", request);

        }

        #endregion
    }
}