﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives
using System;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Helpers;
using SolidFire.Core.Objects;
using SolidFire.Core.Validation;
using SolidFire.Core;
using System.Linq;
using System.Collections.Generic;
using SolidFire.Exceptions;
using SolidFire.Element.Api;
using System.Threading.Tasks;
using System.Threading;
#endregion

namespace SolidFire.Connection
{
    /// <summary>
    /// </summary>

    [Cmdlet(VerbsCommunications.Connect, "SFCluster", DefaultParameterSetName = "PSCred")]
    public class ConnectSFCluster : SFCmdlet
    {
        #region Constants
        /// <summary>
        /// Version to use for discovering the endpoint api versions.
        /// </summary>
        private const float DefaultVersionApi = SolidFireVersionApi.MinVersionApiNumber;

        #endregion

        #region Private Data

        private float? _versionApi;
        private int? _port;
        private int? _timeout;

        #endregion

        #region Parameters

        /// <summary>
        /// </summary>
        [Parameter(Position = 0, ParameterSetName = "PSCred", Mandatory = true, HelpMessage = "Please enter a valid SolidFire cluster or node FQDN or IP address")]
        [Parameter(Position = 0, ParameterSetName = "StringCred", Mandatory = true, HelpMessage = "Please enter a valid SolidFire cluster or node FQDN or IP address")]
        [ValidateLength(0, 100)]
        [ValidateNotNullOrEmpty]
        public new string Target
        {
            get;
            set;
        }

        /// <summary>
        /// </summary>
        [Parameter(Position = 1, ParameterSetName = "PSCred", Mandatory = true, ValueFromPipeline = true, HelpMessage = "Please supply a valid PSCredential. Use Get-Credential.")]
        [ValidateNotNullOrEmpty]
        public PSCredential Credential
        {
            get;
            set;
        }

        /// <summary>
        /// </summary>
        [Parameter(Position = 1, ParameterSetName = "StringCred", Mandatory = true, HelpMessage = "Please enter a valid user name")]
        public string Username
        {
            get;
            set;
        }

        /// <summary>
        /// </summary>
        [Parameter(Position = 2, ParameterSetName = "StringCred", Mandatory = true, HelpMessage = "Please enter a valid password")]
        public string Password
        {
            get;
            set;
        }

        /// <summary>
        /// Indicate whether or not the target address is for a node.
        /// </summary>
        [Parameter(HelpMessage = "Please enter a valid SF node FQDN or IP address")]
        public SwitchParameter Node { get; set; }

        /// <summary>
        /// </summary>
        [Parameter(HelpMessage = "Enter the API Version number")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public float VersionApi
        {
            get
            {
                return _versionApi ?? SolidFireVersionApi.MinVersionApiNumber;
            }
            set
            {
                _versionApi = value;
            }
        }

        [Parameter(HelpMessage = "Enter the Port number")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public int Port
        {
            get
            {
                return _port ?? 0;
            }
            set
            {
                _port = value;
            }
        }

        /// <summary>
        /// This parameter to change the timeout of the http client. Default is 40 seconds.
        /// </summary>
        [Parameter(HelpMessage = "Use this parameter to change the timeout of the http client. Default is 40 seconds.")]
        public int Timeout
        {
            get
            {
                return _timeout ?? 0;
            }
            set
            {
                _timeout = value;
            }
        }

        #endregion

        bool _prevLogRequests;
        bool _prevLogResponses;

        #region Cmdlet Overrides
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            // Give the static Solidfire-Utilities object access to the Powershell SessionState.
            SolidFireUtilities.SessionState = SessionState;

            _prevLogRequests = SolidFireUtilities.LogRequests;
            _prevLogResponses = SolidFireUtilities.LogResponses;

            if (MyInvocation.BoundParameters.ContainsKey("Verbose"))
            {
                SolidFireUtilities.LogResponses = true;
                SolidFireUtilities.LogRequests = true;
            }
        }

        protected override void EndProcessing()
        {
            base.EndProcessing();
            SolidFireUtilities.LogResponses = _prevLogResponses;
            SolidFireUtilities.LogRequests = _prevLogRequests;
        }

        /// <summary>
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            PSCredential objNetworkCredential;

            switch (ParameterSetName)
            {
                // Used in the case of parameter set being used to specify a PSCredential (i.e. Get-Credential) for the cluster or node in the cmdlet parameters.
                case "PSCred":
                    objNetworkCredential = Credential;
                    break;

                // Used in the case of parameter set being specified a user name and password for the cluster or node in the cmdlet parameters.
                case "StringCred":
                    objNetworkCredential = new PSCredential(Username, Password.ConvertToSecureString());
                    break;

                default:
                    throw new ArgumentException("Invalid parameter set.");

            }

            SFConnection connection;

            // create the new connection
            if (_versionApi.HasValue)
            {
                connection = new SFConnection(objNetworkCredential, Target, Node, SolidFireVersionApi.GetByNumber(_versionApi.Value), _port, _timeout);
            }
            else
            {
                connection = new SFConnection(objNetworkCredential, Target, Node, null, _port, _timeout);
            }
            

            try
            {
                // Validate the connection. Intentionally block further processing until this is complete to avoid making calls on non-desired url or version.
                //_validateConnection(connection);
                WriteVerbose(string.Format("Endpoint: {0}", connection.Uri));

                // Set the name
                connection.Name = _getConnectionName(connection.Element);
                // Set the limits
                if (!Node)
                {
                    connection.Limits = connection.Element.GetLimits();
                }
            }
            catch (Exception ex)
            {
                ThrowTerminatingError(new ErrorRecord(ex, "ConnectionError", ErrorCategory.OpenError, null));
            }

            // Were we already connected to this connection? If so, notify user it was previously connected and set SFConnection to this connection
            var existingConnection = SolidFireUtilities.SFConnections.FirstOrDefault(c => c.Equals(connection));
            if (existingConnection != null)
            {
                WriteVerbose(string.Format("Previously connected to '{0}' with address '{1}'.", SolidFireUtilities.SFConnection.Name, SolidFireUtilities.SFConnection.Target));
                WriteVerbose(string.Format("Setting $SFConnection to '{0}'", existingConnection.Name));
                // remove the old connection
                _removeConnection(existingConnection);
            }

            // add this connection to Global list
            _addConnection(connection);

            WriteObject(connection, false);
            WriteVerbose(string.Format("Successfully connected to '{0}' with address '{1}'.", SolidFireUtilities.SFConnection.Name, SolidFireUtilities.SFConnection.Target));

        }

        /// <summary>
        /// Fetch the connetion name from either the cluster or the config depending on whether it's a cluster or node connection.
        /// Set the connection name into the object that ends up in the Global scope as $SFConnection
        /// </summary>
        private string _getConnectionName(SolidFireElement element)
        {
            if (Node)
            {
                var nodeInfo = element.GetConfig();
                return nodeInfo.Config.Cluster.Name;
            }
            else
            {
                var clusterInfo = element.GetClusterInfo();
                return clusterInfo.ClusterInfo.Name;
            }

        }

        /// <summary>
        /// Removes connection from Global variable.
        /// </summary>
        /// <param name="connection">The connection to remove</param>
        private void _removeConnection(SFConnection connection)
        {
            SolidFireUtilities.RemoveSFConnections(new List<SFConnection>() { connection });
        }

        /// <summary>
        /// Adds connection to Global variables for access by future CmdLet calls.
        /// </summary>
        /// <param name="connection">The connecttion to add</param>
        private void _addConnection(SFConnection connection)
        {
            // Record the connection in $SFConnection and add
            // to the list $SFConnections
            var connections = SolidFireUtilities.SFConnections;
            connections.Add(connection);

            // Create the SFConnection variable in the shell session
            SolidFireUtilities.SFConnection = connection;

            // Create the SFConnections variable in the shell session            
            SolidFireUtilities.SFConnections = connections;
        }

        #endregion
    }
}