﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Management.Automation;
using SolidFire.Core.Helpers;
using SolidFire.Core.Objects;
#endregion

namespace SolidFire.Connection
{
    /// <summary>
    /// </summary>
    
    [Cmdlet(VerbsCommunications.Disconnect, "SFCluster", DefaultParameterSetName = "Target")]
    public class DisconnectSFCLuster : PSCmdlet
    {
        #region Private Data

        private string[] _target;
        private string[] _name;

        #endregion

        #region Parameters

        [Parameter(Position = 0, ParameterSetName = "Target", Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, ValueFromRemainingArguments = false, HelpMessage = "Target address of node or cluster to disconnect from")]
        [ValidateNotNullOrEmpty]
        public String[] Target
        {
            get
            {
                return _target;
            }
            set
            {
                _target = value;
            }
        }

        [Parameter(Position = 0, ParameterSetName = "Name", Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, ValueFromRemainingArguments = false, HelpMessage = "Name of a node or cluster to disconnect from")]
        [ValidateNotNullOrEmpty]
        public String[] Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }

        #endregion

        #region Cmdlet Overrides
        /// <summary>
        /// 
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            if (SolidFireUtilities.SFConnection == null)
            {
                var errorRecord =
                    new ErrorRecord(
                        new ItemNotFoundException("You are not currently connected. You require a connection first in order to use the Disconnect-SFCluster cmdlet."),
                        "ConnectionNotFound", ErrorCategory.ObjectNotFound, null);
                SolidFireUtilities.HandleError(errorRecord, this);
                return;
            }

            switch (ParameterSetName)
            {
                case "Target":
                    ProcessByTarget();
                    break;

                case "Name":
                    ProcessByName();
                    break;

                default:
                    throw new ArgumentException("Invalid parameter set.");
            }
        }

        private void ProcessByTarget()
        {
            var connectionsToRemove = new List<SFConnection>();

            // If nothing was specified for the Target then assume we want to disconnect "all".
            if (_target == null)
            {
                foreach (var sfConnection in SolidFireUtilities.SFConnections)
                {
                    if (WhatIfShouldProcessHandler(String.Format(@"User: {0} Connection: {1} {2}", sfConnection.Credential.UserName, sfConnection.Name, sfConnection.Target)))
                    {
                        connectionsToRemove.Add(sfConnection);
                    }
                }
                SolidFireUtilities.RemoveSFConnections(connectionsToRemove);
                _writeOut(connectionsToRemove);
                return; // Return as we are done with disconnections and do not want to process the rest of this method.
            }

            const WildcardOptions options = WildcardOptions.IgnoreCase |
                                                WildcardOptions.Compiled;

            foreach (var target in _target)
            {
                var matchedConnections = new List<SFConnection>();
                var connectionNameFound = false;

                foreach (var sfConnection in SolidFireUtilities.SFConnections)
                {
                    var wildcard = new WildcardPattern(target, options);

                    // If we match on either a node or a cluster (ensure they are not empty either) then add the connection object to list of matched connections for use later on.
                    if ((wildcard.IsMatch(sfConnection.Target) && !string.IsNullOrEmpty(sfConnection.Target)))
                    {
                        if (WhatIfShouldProcessHandler(String.Format(@"User: {0} Target: {1}", sfConnection.Credential.UserName, sfConnection.Target)))
                        {
                            if (!matchedConnections.Contains(sfConnection))
                            {
                                matchedConnections.Add(sfConnection);
                                connectionNameFound = true;
                                break;
                            }
                        }
                        else
                        {
                            connectionNameFound = true; // Set true as the user is only testing using WhatIf.
                            break;
                        }
                    }
                }

                if (matchedConnections.Count > 0)
                {
                    var connectionList = SolidFireUtilities.SFConnections;
                    if (connectionList != null)
                    {
                        foreach (var sfConnection in connectionList)
                        {
                            if (matchedConnections.Contains(sfConnection)) connectionsToRemove.Add(sfConnection);
                        }
                    }
                }

                if (connectionNameFound) continue;

                var errorRecord =
                    new ErrorRecord(
                        new ItemNotFoundException("The specified target " + target + " was not found."),
                        "ClusterConnectionNotFound", ErrorCategory.ObjectNotFound, null);
                SolidFireUtilities.HandleError(errorRecord, this);

            }

            SolidFireUtilities.RemoveSFConnections(connectionsToRemove);
            _writeOut(connectionsToRemove);
        }

        private void ProcessByName()
        {
            var connectionsToRemove = new List<SFConnection>();

            const WildcardOptions options = WildcardOptions.IgnoreCase |
                                                WildcardOptions.Compiled;

            foreach (var name in _name)
            {
                var matchedConnections = new List<SFConnection>();
                var connectionNameFound = false;

                foreach (var sfConnection in SolidFireUtilities.SFConnections)
                {
                    var wildcard = new WildcardPattern(name, options);

                    // If we match on either a node or a cluster (ensure they are not empty either) then add the connection object to list of matched connections for use later on.
                    if ((wildcard.IsMatch(sfConnection.Name) && !string.IsNullOrEmpty(sfConnection.Name)))
                    {
                        if (!matchedConnections.Contains(sfConnection))
                        {
                            matchedConnections.Add(sfConnection);
                            connectionNameFound = true;
                            break;
                        }
                        else
                        {
                            connectionNameFound = true; // Set true as the user is only testing using WhatIf.
                            break;
                        }
                    }
                }

                if (matchedConnections.Count > 0)
                {
                    var connectionList = SolidFireUtilities.SFConnections;
                    if (connectionList != null)
                    {
                        foreach (var sfConnection in connectionList)
                        {
                            if (matchedConnections.Contains(sfConnection)) connectionsToRemove.Add(sfConnection);
                        }
                    }
                }

                if (connectionNameFound) continue;

                var errorRecord =
                    new ErrorRecord(
                        new ItemNotFoundException("The specified name for the connection " + name + " was not found."),
                        "NodeConnectionNotFound", ErrorCategory.ObjectNotFound, null);
                SolidFireUtilities.HandleError(errorRecord, this);

            }

            SolidFireUtilities.RemoveSFConnections(connectionsToRemove);
            _writeOut(connectionsToRemove);

            if (SolidFireUtilities.SFConnections.Count > 0)
            {
                // Set the SFConnection object to the first entry in the list and save the SFConnection PS variable.
                SolidFireUtilities.SFConnection = SolidFireUtilities.SFConnections.First();
            }
        }

        private void _writeOut(List<SFConnection> connectionsRemoved)
        {
            foreach (var conn in connectionsRemoved)
            {
                WriteObject("Disconnected from '" + conn.Name + "' - " + conn.Target);
            }
        }

        protected override void EndProcessing()
        {
            base.EndProcessing();

            // If we have remaining connections, but disconnected a previous "current" connection
            if (SolidFireUtilities.SFConnection == null && SolidFireUtilities.SFConnections.Count > 0)
            {
                if (SolidFireUtilities.SFConnections.First() != null)
                {
                    WriteVerbose(string.Format("Assigning top remaining connection ({0}) to current SFConnection.", SolidFireUtilities.SFConnections[0].Target));
                    // Pop the current top connection left over in SFConnections to the "current" connection at SFConnection.
                    SolidFireUtilities.SFConnection = SolidFireUtilities.SFConnections.First();
                }
            }
        }

        protected bool WhatIfShouldProcessHandler(string message)
        {
            return (ShouldProcess(message, this.GetType().Name));
        }

        #endregion
    }
}