﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives
using SolidFire.Core.Helpers;
using SolidFire.Element;
using SolidFire.Element.Api;
using System;
using System.Globalization;
using System.Management.Automation;
using System.Net;
using System.Security;
using SolidFireLimits = SolidFire.Element.Api.GetLimitsResult;
#endregion

namespace SolidFire.Core.Objects
{
    public class SFConnection 
    {
        public SFConnection(PSCredential credential, string target, bool node, VersionApi apiVersion, int? port = null, int? timeout = null)
        {
            Credential = credential;
            _target = target;
            _versionApi = apiVersion;
            _port = node ? 442 : port;
            Timeout = timeout;
            _makeElement();
        }

        #region Public Properties

        private string _target;
        /// <summary>
        ///  The host this connection points at.
        /// </summary>
        public string Target { get { return _target; } }

        // The name of this connection. Usually set during execution of the Connect-SFCluster cmdlet.
        public string Name { get; set; }

        private int? _port;
        // The port this connection uses in the URI. 
        public int? Port
        {
            get { return _port; }
        }

        /// <summary>
        /// Changes the Port this connection uses.
        /// Upon being changed, the underlying SolidFireElement object will
        /// be re-created with the new port in its endpoint.
        /// </summary>
        /// <param name="port"></param>
        public void SetPort(int? port)
        {
            if (_port != port)
            {
                _port = port;
                _makeElement();
            }
        }

        private VersionApi _versionApi;

        /// <summary>
        /// Changes the Version this connection is pointing to. 
        /// Upon being changed, the underlying SolidFireElement object will 
        /// be re-created with the new version number in its endpoint.
        /// </summary>
        /// <param name="versionApi"></param>
        public void SetVersionApi(VersionApi versionApi)
        {
            if (versionApi == null)
            {
                throw new ArgumentNullException("versionApi");
            }

            if (_versionApi != versionApi)
            {
                _versionApi = versionApi;
                _makeElement();
            }
        }

        public string VersionApiName => SolidFireVersionApi.GetByNumber(_element.GetConnectionVersion()).Name;

        public string VersionApiNumber => _element.GetConnectionVersion().ToString("F1", CultureInfo.InvariantCulture);

        /// <summary>
        /// Is this connection pointing to a Node?
        /// </summary>
        public bool Node => _port == 442;

        /// <summary>
        /// The Uri being used by the underlying SolidFire element dispatcher.
        /// </summary>
        public Uri Uri => _element.GetUri();

        /// <summary>
        /// Makes a new SolidFireElement object for use in requests/responses. This should only be changed
        /// internally and only when VersionApi or Port changes. 
        /// </summary>
        private void _makeElement()
        {
            _element = ElementFactory.Create(
                target: Target,
                credential: Credential.GetNetworkCredential(),
                version: _versionApi != null ? _versionApi.Number.ToString("F1", CultureInfo.InvariantCulture) : null,
                port: _port.HasValue ? _port : Node ? (int?)442 : null
                );

            _element.OnResponse += element_OnResponse;
            _element.OnRequest += element_OnRequest;
        }
        
    void element_OnResponse(string content)
        {
            if (SolidFireUtilities.LogResponses)
            {
                Console.WriteLine("RESPONSE: " + content);
            }
        }

        void element_OnRequest(string content)
        {
            if (SolidFireUtilities.LogRequests)
            {
                Console.WriteLine("REQUEST: " + content);
            }
        }

        public int RequestCount // Exposes the underlying SolidFireElement request count.
        { get { return _element.GetRequestCount(); } }

        public PSCredential Credential { get; set; }

        public SolidFireLimits Limits { get; set; }

        /// <summary>
        /// Timeout in Seconds. If set and different from the default, this value will determine the timeout of the underlying http client.
        /// </summary>
        public int? Timeout { get; set; }

        private SolidFireElement _element;
        /// <summary>
        /// The underlying SolidFireElement dispatcher.
        /// </summary>
        public SolidFireElement Element
        {
            get { return _element; }
        }
        
        #endregion

        public object Clone()
        {
            return MemberwiseClone();
        }

        public override bool Equals(object obj)
        {
            if (obj is SFConnection)
            {
                var conn = (SFConnection)obj;
                return Name == conn.Name;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return Name.GetHashCode();
        }
    }
}