﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Validation;
using SolidFire.Core;
using System.Linq;
using SolidFire.Element.Api;
#endregion

namespace SolidFire.Drive.Add
{
    /// <summary>
    /// AddDrives is used to add one or more available drives to the cluster enabling the drives to host a portion of the cluster's data.
    /// When a new drive is made available to the cluster, either through a newly added node or by being inserted into an existing
    /// node, it is marked as "available" and must be added via AddDrives before it is utilized. Use the ListDrives method to display
    /// drives that are "available" to be added. When multiple drives are being added, it is more efficient to add them in a single
    /// AddDrives method rather than multiple individual methods with a single drive. This reduces the amount of data balancing that
    /// must occur to even out the storage load on the cluster.
    /// 
    /// When adding a drive, the system automatically determines the "type" of drive it should be.
    /// 
    /// The method returns immediately, however it may take some period of time for the data in the cluster to be fully rebalanced
    /// using the newly added drives. As the new drive(s) are syncing on the system, you can use the ListSyncJobs method to see
    /// how the drive(s) are being rebalanced and the progress of adding the new drive.
    /// 
    /// </summary>
    
    [Cmdlet(VerbsCommon.Add, "SFDrive", ConfirmImpact = ConfirmImpact.Medium)]
    public class AddSFDrive : SFCmdlet
    {
        #region Private Data

        private List<NewDrive> _newDrivesToAdd = new List<NewDrive>();

        #endregion

        #region Parameters
        [Parameter(Mandatory = true, Position = 0, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The list of drive IDs to add to the cluster")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public Int64[] DriveID { get; set; }
        #endregion

        #region Cmdlet Overrides
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            CheckConnection();
        }

        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            _newDrivesToAdd.AddRange(DriveID.Select(d => new NewDrive() { DriveID = d }));

        }

        protected override void EndProcessing()
        {
            base.EndProcessing();
            var request = new AddDrivesRequest();

            request.Drives = _newDrivesToAdd.Distinct().ToArray();

            var objsFromAPI = SendRequest<AddDrivesResult>("AddDrives", request);
            foreach (var obj in objsFromAPI)
            {
                WriteObject(obj.Result, true);
            }
        }

        #endregion

    }
}