﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives
using System;
using System.Collections.Generic;
using System.Management.Automation;
using System.Management.Automation.Runspaces;
using System.Runtime.InteropServices;
using System.Threading;
using SolidFire.Core.Helpers;
using SolidFire.Core.Validation;
using SolidFire.Core;
using SolidFire.Core.Objects;
using SolidFire.Element.Api;
#endregion

namespace SolidFire.Node.Set
{
    /// <summary>
    /// Used to set the network configuration for the node
    /// </summary>
    
    [Cmdlet(VerbsCommon.Set, "SFNetworkConfig", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.High)]
    public class SetSFNetworkConfig : SFCmdlet
    {
        #region Private Data
        private Int64? mtu;
        #endregion

        #region Parameters

        /// <summary>
        /// The interface to set settings on. Must be one of "Bond1G" | "Bond10G".
        /// Note that this explicitly disallows use on management nodes, which is probably a good thing.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The interface to set on. Must be one of 'Bond1G' | 'Bond10G'. Changing the BondMode on a node can cause a loss of network connectivity. Caution should be taken.")]
        [ValidateSet("Bond10G", "Bond1G", "Eth0", "Eth1", "Eth2", "Eth3", "Lo")]
        public string Network { get; set; }

        /// <summary>
        /// The Bond mode to set.
        /// </summary>
        [Parameter(Position = 1, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The bond mode to set for the interface")]
        [ValidateSet("ActivePassive", "ALB", "LACP", IgnoreCase = true)]
        public string BondMode { get; set; }

        /// <summary>
        /// The IP address for the interface. If this changes, and this is the interface you are issuing your API calls against, you're going to drop!
        /// TODO: Handle this much more gracefully.
        /// </summary>
        [Parameter(Position = 2, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The IP address to set on the interface. Changing the IP Address on a node can cause a loss of network connectivity. Be prepared to reconnect if this command times out.")]
        [ValidatePattern(SolidFireValidations.IPAddress)]
        public string Address { get; set; }

        /// <summary>
        /// The netmask for the interface. This will not validate that your netmask / gateway make sense together!
        /// </summary>
        [Parameter(Position = 3, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The Netmask to use for the interface")]
        [ValidatePattern(SolidFireValidations.IPAddress)]
        public string Netmask { get; set; }

        /// <summary>
        /// The gateway for the interface.
        /// </summary>
        [Parameter(Position = 4, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The gateway address to set on the interface")]
        [ValidatePattern(SolidFireValidations.IPAddress)]
        public string Gateway { get; set; }

        /// <summary>
        /// The MTU for the interface.
        /// </summary>
        [Parameter(Position = 5, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The MTU to set on the interface.")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public Int64 Mtu
        {
            // this 0 value will be caught in BeginProcessing.
            get { return mtu.HasValue ? mtu.Value : 0; }
            set { mtu = value; }
        }

        [Parameter(Position = 6, Mandatory = false, HelpMessage = "Add -DisconnectAfterRequest to disconnect the SFConnection after the request is sent. It will not wait for a response.")]
        public SwitchParameter DisconnectAfterRequest { get; set; }

        /// <summary>
        /// The DNS servers to use.
        /// </summary>
        [Parameter(Position = 6, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The DNS nameservers to set on the interface")]
        [ValidatePattern(SolidFireValidations.IPAddress)]
        public string[] DnsNameservers { get; set; }

        /// <summary>
        /// The list of search domains to use.
        /// It would be nice to validate these as actual valid domains!
        /// </summary>
        [Parameter(Position = 7, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The DNS search domains")]
        public string[] DnsSearch { get; set; }

        /// <summary>
        /// The interface status to set. Must be one of UpAndRunning, Up, or Down
        /// </summary>
        [Parameter(Position = 8, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The interface status ('UpAndRunning' | 'Down' | 'Up')")]
        [ValidateSet("UpAndRunning", "Up", "Down", IgnoreCase = true)]
        public string Status { get; set; }

        #endregion

        #region Cmdlet Overrides
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            if (mtu.HasValue)
            {
                // If the MTU on the 10G is <9000, you're going to have exciting intra-cluster communication issues.
                if (Network == "Bond10G" && Mtu < 9000)
                {
                    var errDetails = new ErrorDetails("Error: MTU must be >= 9000 on the 10G network");
                    var errRecord = new ErrorRecord(new Exception(), errDetails.Message, ErrorCategory.InvalidArgument, this)
                    {
                        ErrorDetails = errDetails
                    };
                    ThrowTerminatingError(errRecord);
                }
                //this would cause an error from the cluter itself, but might as well handle it similarly to the 10G.
                else if (Network == "Bond1G" && Mtu < 1500)
                {
                    var errDetails = new ErrorDetails("Error: MTU must be >= 1500 on the 1G network");
                    var errRecord = new ErrorRecord(new Exception(), errDetails.Message, ErrorCategory.InvalidArgument, this)
                    {
                        ErrorDetails = errDetails
                    };
                    ThrowTerminatingError(errRecord);
                }
            }

            CheckConnection(endPoint: SolidFireUtilities.SFEndPoint.Node);
        }

        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            var request = new SetNetworkConfigRequest();
            request.Network = new NetworkParams();
            var config = new NetworkConfigParams();
            var intf = new Dictionary<string, object> { };
            if (!String.IsNullOrEmpty(BondMode))
            {
                config.BondMode = BondMode;
                WriteWarning("Changing the BondMode on a node can cause a loss of network connectivity. Caution should be taken.");
            }
            if (!String.IsNullOrEmpty(Address))
            {
                config.Address = Address;
                WriteWarning("Changing the IP Address on a node can cause a loss of network connectivity. Be prepared to reconnect if this command times out.");
            }
            if (!String.IsNullOrEmpty(Netmask)) {
                config.Netmask = Netmask;
            }
            if (!String.IsNullOrEmpty(Gateway)) {
                config.Gateway = Gateway;
            }
            if (!String.IsNullOrEmpty(Status)) {
                config.Status = Status; 
            }
            if (mtu.HasValue) {
                config.Mtu = Mtu.ToString();            
            }

            if (DnsNameservers != null && DnsNameservers.Length != 0)
            {
                config.DnsNameservers = string.Join(", ", DnsNameservers);
            }
            if (DnsSearch != null && DnsSearch.Length != 0)
            {
                config.DnsSearch = string.Join(", ", DnsSearch);
            }

            switch (Network)
            {
                case "Bond1G":
                    request.Network.Bond1G = config;
                    break;
                case "Bond10G":
                    request.Network.Bond10G = config;
                    break;
                case "Eth0":
                    request.Network.Eth0 = config;
                    break;
                case "Eth1":
                    request.Network.Eth1 = config;
                    break;
                case "Eth2":
                    request.Network.Eth2 = config;
                    break;
                case "Eth3":
                    request.Network.Eth3 = config;
                    break;
                case "Lo":
                    request.Network.Lo = config;
                    break;
            }

            if (DisconnectAfterRequest.IsPresent)
            {
                var connectionsToRemove = new List<SFConnection>();
                WriteWarning("The following SFConnections will be disconnected after this SetNetworkConfig request is sent: ");
                if (SFConnection != null)
                {
                    foreach (var conn in SFConnection)
                    {
                        _sendRequest(conn, request, ref connectionsToRemove);
                    }
                }
                else if (Target != null)
                {
                    foreach (var target in Target)
                    {
                        var conn = SolidFireUtilities.GetSFConnection(this, target);
                        _sendRequest(conn, request, ref connectionsToRemove);
                    }
                }
                SolidFireUtilities.RemoveSFConnections(connectionsToRemove);
            }
            else
            {
                var objsFromAPI = SendRequest<SetNetworkConfigResult>("SetNetworkConfig", request, false);
                foreach (var obj in objsFromAPI)
                {
                    WriteObject(obj.Result.Network, true);
                }
            }
        }

        private void _sendRequest(SFConnection conn, SetNetworkConfigRequest request, ref List<SFConnection> connectionsToRemove)
        {
            WriteWarning(conn.Name);
            conn.Element.SetNetworkConfigAsync(request, CancellationToken.None); // This is intentionally not awaited because we don't care to receive the response   
            connectionsToRemove.Add(conn);
        }

        #endregion
    }

}