﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
#endregion

namespace SolidFire.Volume.Add
{
    /// <summary>
    /// AddInitiatorsToVolumeAccessGroup is used to add initiators to a specified volume access group.
    /// The accepted format of an initiator IQN: iqn.yyyy-mm where y and m are digits, followed by text which must only contain digits,
    /// lower-case alphabetic characters, a period (.), colon (:) or dash (-).
    /// </summary>
    
    [Cmdlet(VerbsCommon.Add, "SFInitiatorToVolumeAccessGroup", ConfirmImpact = ConfirmImpact.Medium)]
    public class AddSFInitiatorToVolumeAccessGroup : SFCmdlet
    {

        private Pipeline pipeline;

        #region Parameters

        /// <summary>
        /// The ID of the volume access group to add the initiator.
        /// </summary>
        [Parameter(Mandatory = true, Position = 0, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, ValueFromRemainingArguments = false, HelpMessage = "Please enter a volume access group ID or list of volume access group IDs")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public Int64 VolumeAccessGroupID { get; set; }

        /// <summary>
        /// List of initiators (IQNs and WWPNs) to add to the volume access group.
        /// </summary>
        [Parameter(Mandatory = true, Position = 1, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "List of initiator names (IQNs and WWPNs) to include in the volume access group")]
        // Validation Pattern is too restrictive [ValidatePattern(SolidFireValidations.iSCSIInitiator)]
        public string[] Initiators { get; set; }

        #endregion

        #region CmdLet Overrides
        /// <summary>
        /// Implementation for checking the status of the connection and 
        /// compatibility before allowing the cmdlet to continue.
        /// </summary>
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            CheckConnection();
            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
        }

        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFVolumeAccessGroup");
            command.Parameters.Add("VolumeAccessGroupID", VolumeAccessGroupID);
            if (SFConnection != null)
            {
                command.Parameters.Add("SFConnection", SFConnection);
            }
            else if (Target != null)
            {
                command.Parameters.Add("Target", Target);
            }
            pipeline.Commands.Add(command);
            WriteObject(pipeline.Invoke(), true);
            pipeline = null;
        }

        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            // Parameters to pass into the AddInitiatorsToVolumeAccessGroup API call; 
            // volumeAccessGroupID and initiators are required parameters.
            var request = new AddInitiatorsToVolumeAccessGroupRequest();
            request.Initiators = Initiators;
            request.VolumeAccessGroupID = VolumeAccessGroupID;
            SendRequest<ModifyVolumeAccessGroupResult>("AddInitiatorsToVolumeAccessGroup", request);
        }

        #endregion
    }
}