﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Validation;
using SolidFire.Core;
using SolidFire.Element.Api;
using System.Linq;
using System.Management.Automation.Runspaces;
using System;
using SolidFire.Exceptions;
using SolidFire.Core.Helpers;
#endregion

namespace SolidFire.Volume.Get
{
    /// <summary>
    /// Used to change the intervals at which a scheduled snapshot occurs. This allows for adjustment to the
    /// snapshot frequency and retention.
    /// </summary>
    
    [Cmdlet(VerbsCommon.Set, "SFSchedule", ConfirmImpact = ConfirmImpact.Medium)]
    public class SetSFSchedule : SFCmdlet, IDynamicParameters
    {
        #region Private Data
        
        // Prepare the DynamicParameter classes so they're ready when Frequency is selected.
        private TimeIntervalDynamicParameters _contextTimeInterval = new TimeIntervalDynamicParameters();
        private DaysOfMonthDynamicParameters _contextDaysOfMonth = new DaysOfMonthDynamicParameters();
        private DaysOfWeekDynamicParameters _contextDaysOfWeek = new DaysOfWeekDynamicParameters();

        private List<long> _processedIDs = new List<long>();

        private Pipeline _pipeline;

        #endregion

        #region Parameters

        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Unique ID of the schedule.")]
        public long[] ScheduleID { get; set; }

        [Parameter(Position = 1, Mandatory = false, HelpMessage = "Volume IDs to be included in the resulting snapshots. VolumeID values will replace ScheduleInfo.Volume values if an existing Schedule is piped in.")]
        public long[] VolumeID { get; set; }

        [Parameter(Position = 2, Mandatory = false, HelpMessage = "Unique name for the schedule.")]
        public string ScheduleName { get; set; }

        [Parameter(Position = 3, Mandatory = false, HelpMessage = "Used to indicate the frequency at which the snapshot will be made.")]
        [ValidateSet("TimeInterval", "DaysOfWeek", "DaysOfMonth")]
        public string Frequency { get; set; }

        private bool? _toBeDeleted;
        [Parameter(Position = 4, Mandatory = false, HelpMessage = "Indicates if the schedule is marked for deletion.")]
        public bool ToBeDeleted { get { return _toBeDeleted.HasValue ? _toBeDeleted.Value : false; } set { _toBeDeleted = value; } }

        private bool? _paused;
        [Parameter(Position = 5, Mandatory = false, HelpMessage = "Use to pause the schedule.")]
        public bool Paused { get { return _paused.HasValue ? _paused.Value : false; } set { _paused = value; } }

        private bool? _runNextInterval;
        [Parameter(Position = 6, Mandatory = false, HelpMessage = "Choose to run this Schedule the next time the scheduler is active.")]
        public bool RunNextInterval { get { return _runNextInterval.HasValue ? _runNextInterval.Value : false; } set { _runNextInterval = value; } }

        [Parameter(Position = 7, Mandatory = false, HelpMessage = "Name of the new snapshot.")]
        public string SnapshotName { get; set; }

        private bool? _recurring = true;
        [Parameter(Position = 8, Mandatory = false, HelpMessage = "Indicates if the schedule will be recurring or not. Default is true.")]
        public bool Recurring { get { return _recurring.HasValue ? _recurring.Value : false; } set { _recurring = value; } }

        private bool? _enableRemoteReplication;
        [Parameter(Position = 9, Mandatory = false, HelpMessage = " Include snapshots in replication when paired. Default is false.")]
        public bool EnableRemoteReplication { get { return _enableRemoteReplication.HasValue ? _enableRemoteReplication.Value : false; } set { _enableRemoteReplication = value; } }

        private ulong? _retentionDays;
        [Parameter(Position = 10, Mandatory = false, HelpMessage = "Number of days to retain snapshots created by this schedule.")]
        public ulong RetentionDays { get { return _retentionDays.HasValue ? _retentionDays.Value : 0; } set { _retentionDays = value; } }

        private ulong? _retentionHours;
        [Parameter(Position = 11, Mandatory = false, HelpMessage = "Number of hours to retain snapshots created by this schedule.")]
        public ulong RetentionHours { get { return _retentionHours.HasValue ? _retentionHours.Value : 0; } set { _retentionHours = value; } }

        private ulong? _retentionMinutes;
        [Parameter(Position = 12, Mandatory = false, HelpMessage = "Number of minutes to retain snapshots created by this schedule.")]
        [ValidateRange(0, 59)]
        public ulong RetentionMinutes { get { return _retentionMinutes.HasValue ? _retentionMinutes.Value : 0; } set { _retentionMinutes = value; } }

        [Parameter(Position = 13, Mandatory = false, HelpMessage = "Date/Time after which the schedule will be run.")]
        [SFValidatePattern(SolidFireValidations.ISO8601DateTimeOrEmptyString, "The argument is not in the format of an ISO-8601 Date String.")]
        public string StartingDate { get; set; }

        #endregion

        #region Cmdlet Overrides

        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            CheckConnection(minVersionNumber: 8.0f);
            _pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
        }

        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            foreach (var scheduleID in ScheduleID)
            {
                var scheduleBeforeModify = getSchedule(scheduleID);

                if (!string.IsNullOrEmpty(ScheduleName))
                {
                    scheduleBeforeModify.Name = ScheduleName;
                }
                if (_toBeDeleted.HasValue)
                {
                    scheduleBeforeModify.ToBeDeleted = _toBeDeleted.Value;
                }
                if (_paused.HasValue)
                {
                    scheduleBeforeModify.Paused = _paused.Value;
                }
                if (_recurring.HasValue)
                {
                    scheduleBeforeModify.Recurring = _recurring.Value;
                }
                if (_runNextInterval.HasValue)
                {
                    scheduleBeforeModify.RunNextInterval = _runNextInterval.Value;
                }
                if (!string.IsNullOrEmpty(StartingDate))
                {
                    scheduleBeforeModify.StartingDate = StartingDate;
                }
                if ((_retentionDays != null && _retentionDays.HasValue) || _retentionHours.HasValue || _retentionMinutes.HasValue)
                {
                    scheduleBeforeModify.ScheduleInfo.Retention = VolumeUtilities.BuildTimeString(RetentionDays, RetentionHours, RetentionMinutes);
                }
                if (_enableRemoteReplication.HasValue)
                {
                    scheduleBeforeModify.ScheduleInfo.EnableRemoteReplication = _enableRemoteReplication.Value;
                }
                if (!string.IsNullOrEmpty(SnapshotName))
                {
                    scheduleBeforeModify.ScheduleInfo.SnapshotName = SnapshotName;
                }
                if (VolumeID != null && VolumeID.Count() > 0)
                {
                    scheduleBeforeModify.ScheduleInfo.VolumeIDs = VolumeID.Select(v => v).ToArray();
                }

                // Perform dynamic property assignment based on Frequency
                switch (Frequency)
                {
                    case "TimeInterval":
                        var timeInterval = new TimeIntervalFrequency()
                        {
                            Hours = (long)_contextTimeInterval.TimeIntervalHours,
                            Days = (long)_contextTimeInterval.TimeIntervalDays,
                            Minutes = (long)_contextTimeInterval.TimeIntervalMinutes
                        };
                        scheduleBeforeModify.Frequency = timeInterval;
                        break;
                    case "DaysOfWeek":
                        var daysOfWeek = new DaysOfWeekFrequency()
                        {
                            Hours = (long)_contextDaysOfWeek.TimeOfDayHour,
                            Minutes = (long)_contextDaysOfWeek.TimeOfDayMinute
                        };
                        var weekdays = new List<Weekday>();
                        foreach (var day in _contextDaysOfWeek.DaysOfWeekDays)
                        {
                            weekdays.Add((Weekday)Enum.Parse(typeof(Weekday), day));
                        }
                        daysOfWeek.Weekdays = weekdays.ToArray();
                        scheduleBeforeModify.Frequency = daysOfWeek;
                        break;
                    case "DaysOfMonth":
                        var daysOfMonth = new DaysOfMonthFrequency()
                        {
                            Hours = (long)_contextDaysOfMonth.TimeOfDayHour,
                            Minutes = (long)_contextDaysOfMonth.TimeOfDayMinute,
                            Monthdays = _contextDaysOfMonth.DaysOfMonthDays.Select(d => (long)d).ToArray()
                        };
                        scheduleBeforeModify.Frequency = daysOfMonth;
                        break;
                };

                var request = new ModifyScheduleRequest();
                request.Schedule = scheduleBeforeModify;

                SendRequest<ModifyScheduleResult>("ModifySchedule", request);

                _processedIDs.Add(scheduleID);
            }
        }

        private Schedule getSchedule(long scheduleID)
        {
            var volPipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
            var command = new Command("Get-SFSchedule");
            command.Parameters.Add("ScheduleID", scheduleID);
            if (SFConnection != null)
            {
                command.Parameters.Add("SFConnection", SFConnection);
            }
            else if (Target != null)
            {
                command.Parameters.Add("Target", Target);
            }
            volPipeline.Commands.Add(command);
            var psObj = volPipeline.Invoke().FirstOrDefault();
            volPipeline = null;

            if (psObj == null)
            {
                var message = string.Format("Unable to find Schedule with ID {0}. Cannot continue with Schedule modification.", scheduleID);
                var errRecord = new ErrorRecord(new RequestException(message), message, ErrorCategory.InvalidData, this);

                SolidFireUtilities.HandleError(errRecord, this, true);
            }
            return (Schedule)psObj.BaseObject;

        }

        /// <summary>
        /// This is necessary to implement IDynamicParameters. It hands back the corresponding parameter set
        /// based on the value in the Frequence parameter.
        /// </summary>
        /// <returns>DynamicParameter class with parameters for the selected Frequency</returns>
        public object GetDynamicParameters()
        {
            switch (Frequency)
            {
                case "TimeInterval":
                    return _contextTimeInterval;
                case "DaysOfMonth":
                    return _contextDaysOfMonth;
                case "DaysOfWeek":
                    return _contextDaysOfWeek;
            }
            return null;
        }

        protected override void EndProcessing()
        {
            base.EndProcessing();
            if (_processedIDs.Count > 0)
            {
                var command = new Command("Get-SFSchedule");
                command.Parameters.Add("ScheduleID", _processedIDs);
                if (SFConnection != null)
                {
                    command.Parameters.Add("SFConnection", SFConnection);
                }
                else if (Target != null)
                {
                    command.Parameters.Add("Target", Target);
                }
                _pipeline.Commands.Add(command);
                WriteObject(_pipeline.Invoke(), true);
                _processedIDs.Clear();
            }
            _pipeline = null;
        }
        #endregion
    }
}