﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Management.Automation;
using SolidFire.Core.Helpers;
using SolidFire.Core.Validation;
using SolidFire.Exceptions;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Core.Objects;
using SolidFire.Element.Api;
using ElementVolume = SolidFire.Element.Api.Volume;
#endregion

namespace SolidFire.Volume.Set
{
    /// <summary>
    /// SetSFVolume is used to modify settings on an existing volume. Modifications can be made to one volume at a time and changes
    /// take place immediately.
    /// 
    /// QoS values that are not specified when a volume is modified is changed to the Default values listed in the QoS table on page 25.
    /// This occurs even if QoS values were previously supplied. It is highly recommended that all QoS values be specified when modifying
    /// a volume. Default values for a volume can be found by running the GetDefaultQoS method.
    /// 
    /// Extending the size of a volume that is being replicated should be done in order. The target, "Replication Target", volume should first
    /// be increased in size, then the source, "Read / Write", volume can be resized. It is recommended that both the target and the source
    /// volumes be the same size.    
    /// </summary>

    [Cmdlet(VerbsCommon.Set, "SFVolume", DefaultParameterSetName = "q", SupportsShouldProcess = true,
        ConfirmImpact = ConfirmImpact.High)]
    public class SetSFVolume : SFCmdlet
    {
        #region Private Data

        /// <summary>
        /// Member to track the account id 
        /// </summary>
        private long? accountId;

        /// <summary>
        /// Member for saving the optional minimum iops setting. Implemented
        /// as a nullable to help decide if the setting needs to be sent
        /// to the api call as JSON data.
        /// </summary>
        private int? minIOPS;

        /// <summary>
        /// Member for saving the optional maximum iops setting. Implemented
        /// as a nullable to help decide if the setting needs to be sent
        /// to the api call as JSON data.
        /// </summary>
        private int? maxIOPS;

        /// <summary>
        /// Member for saving the optional burst iops setting. Implemented
        /// as a nullable to help decide if the setting needs to be sent
        /// to the api call as JSON data.
        /// </summary>
        private int? burstIOPS;

        /// <summary>
        /// Member for saving the optional total size setting for the volume.
        /// </summary>
        private long? totalSize;

        private int? burstTime;

        private int? qosPolicyID;

        private bool? associateWithQoSPolicy;

        private List<long> processedIDs = new List<long>();

        private Pipeline pipeline;

        #endregion

        #region Parameters

        /// <summary>
        /// List of VolumeID's to be modified.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "VolumeID(s) for the volume(s) to be modified")]
        public long[] VolumeID { get; set; }

        /// <summary>
        /// AccountID to which the volume is reassigned. If none is specified, the previous account name is used.
        /// </summary>
        [Parameter(Position = 1, Mandatory = false, ValueFromPipeline = false,
            HelpMessage = "Account ID to which the volume is reassigned")]
        public long AccountID
        {
            get { return accountId.HasValue ? accountId.Value : 0; }
            set { accountId = value; }
        }

        /// <summary>
        /// Access allowed for the volume. Possible values:
        ///     readOnly: Only read operations are allowed.
        ///     readWrite: Reads and writes are allowed.
        ///     locked: No reads or writes are allowed.
        ///     If not specified, the access value does not change.
        ///     replicationTarget: Identify a volume as the target volume for a paired set of volumes. 
        ///     If the volume is not paired, the access status is locked. 
        ///     If a value is not specified, the access value does not change.
        /// </summary>
        [Parameter(Position = 2, Mandatory = false, ValueFromPipeline = false,
            HelpMessage = "Access allowed for the volume")]
        [ValidateSet("readOnly", "readWrite", "locked", "replicationTarget", IgnoreCase = true)]
        public string Access { get; set; }

        /// <summary>
        /// Attributes to save with the volume.
        /// </summary>
        [Parameter(Position = 3, Mandatory = false, ValueFromPipeline = false, HelpMessage = "Attributes to set")]
        public Hashtable Attributes { get; set; }

        /// <summary>
        /// Identify the time at which the volume was created.
        /// </summary>
        [Parameter(Position = 4, Mandatory = false, ValueFromPipeline = false,
            HelpMessage = "Identify the time at which the volume was created")]
        public string SetCreateTime { get; set; }

        /// <summary>
        /// Minimum iops to set for the volume.
        /// </summary>
        [Parameter(Position = 5, Mandatory = false, ValueFromPipeline = false, HelpMessage = "The desired minimum 4KB IOPS to guarantee")]
        [Parameter(ParameterSetName = "q")]
        public int MinIOPS
        {
            get { return minIOPS ?? -1; }
            set { minIOPS = value; }
        }

        /// <summary>
        /// Maximum iops to set for the volume.
        /// </summary>
        [Parameter(Position = 6, Mandatory = false, ValueFromPipeline = false, HelpMessage = "The desired maximum 4KB IOPS to guarantee")]
        [Parameter(ParameterSetName = "q")]
        public int MaxIOPS
        {
            get { return maxIOPS ?? -1; }
            set { maxIOPS = value; }
        }

        /// <summary>
        /// Burst iops to set for the volume.
        /// </summary>
        [Parameter(Position = 7, Mandatory = false, ValueFromPipeline = false, HelpMessage = "The maximum 4KB IOPS allowed for a short time")]
        [Parameter(ParameterSetName = "q")]
        public int BurstIOPS
        {
            get { return burstIOPS ?? -1; }
            set { burstIOPS = value; }
        }

        [Parameter(Position = 8, Mandatory = false, ValueFromPipeline = false, HelpMessage = "The maximum 4KB IOPS allowed for a short time")]
        [Parameter(ParameterSetName = "q")]
        public int BurstTime
        {
            get { return burstTime ?? -1; }
            set { burstTime = value; }
        }

        /// <summary>
        /// Total size of the volume, in bytes. Size is rounded up to the nearest 1MB size.
        /// This value interpretation can be in GB or GiB by using the GB or GiB switch.
        /// </summary>
        [Parameter(Position = 10, Mandatory = false, ValueFromPipeline = false,
            HelpMessage = "New size of the volume in GB. You can only increase a volumes size")]
        public long TotalSize
        {
            get { return totalSize ?? -1; }
            set { totalSize = value; }
        }

        [Parameter(Mandatory = false, Position = 5, HelpMessage = "The ID for the policy whose QoS settings should be applied to the specified volumes.")]
        [Parameter(ParameterSetName = "p")]
        public int QoSPolicyID
        {
            get { return qosPolicyID ?? -1; }
            set { qosPolicyID = value; }
        }

        [Parameter(Mandatory = false, Position = 6, HelpMessage = "Associate the volume with the specified QoSPolicyID.")]
        [Parameter(ParameterSetName = "p")]
        public bool AssociateWithQoSPolicy {
            get { return associateWithQoSPolicy ?? true; }
            set { associateWithQoSPolicy = value; }
        }

        /// <summary>
        /// Switch to specify interpreting the Total Size parameter as GiB instead of Bytes.
        /// </summary>
        [Parameter(Mandatory = false, Position = 11, ValueFromPipeline = false, ValueFromPipelineByPropertyName = false,
            HelpMessage = "Switch used to interpret total size as GB")]
        public SwitchParameter GB { get; set; }

        /// <summary>
        /// Switch to specify interpreting the Total Size parameter as GiB instead of Bytes.
        /// </summary>
        [Parameter(Mandatory = false, Position = 12, ValueFromPipeline = false, ValueFromPipelineByPropertyName = false,
            HelpMessage = "Switch used to interpret total size as GiB")]
        public SwitchParameter GiB { get; set; }

        #endregion

        #region Cmdlet Overrides

        /// <summary>
        /// Implementation for validating GB and GiB switches, checking the status 
        /// of the connection, and compatibility before allowing the cmdlet to continue.
        /// </summary>
        protected override void BeginProcessing()
        {
            pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();

            base.BeginProcessing();

            // GB and GiB are mutually exclusive switches.
            if (GB.IsPresent && GiB.IsPresent)
            {
                var errDetails = new ErrorDetails("GB and GiB are mutually exclusive switches");
                var errRecord = new ErrorRecord(new ArgumentException(), errDetails.Message,
                    ErrorCategory.InvalidArgument, this);

                SolidFireUtilities.HandleError(errRecord, this);
            }

            // TotalSize must be specified with GB
            if (!totalSize.HasValue && GB.IsPresent)
            {
                var errDetails = new ErrorDetails("Total Size must be specified when using the GB switch parameter");
                var errRecord = new ErrorRecord(new ArgumentException(), errDetails.Message,
                    ErrorCategory.InvalidArgument, this);

                SolidFireUtilities.HandleError(errRecord, this);
            }

            // TotalSize must be specified with GiB
            if (!totalSize.HasValue && GiB.IsPresent)
            {
                var errDetails = new ErrorDetails("Total Size must be specified when using the GiB switch parameter");
                var errRecord = new ErrorRecord(new ArgumentException(), errDetails.Message,
                    ErrorCategory.InvalidArgument, this);

                SolidFireUtilities.HandleError(errRecord, this);
            }

            CheckConnection();

        }


        /// <summary>
        /// Implementation for processing the execution of the cmdlet.
        /// </summary>
        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            foreach (var vId in VolumeID)
            {
                var volBeforeModify = getVolumeById(vId);
                if (volBeforeModify != null)
                {
                    processVolumeChange(volBeforeModify);
                }
            }
        }
        

        private ElementVolume getVolumeById(long id)
        {
            var volPipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
            var command = new Command("Get-SFVolume");
            command.Parameters.Add("VolumeID", id);
            if (SFConnection != null)
            {
                command.Parameters.Add("SFConnection", SFConnection);
            }
            else if (Target != null)
            {
                command.Parameters.Add("Target", Target);
            }
            volPipeline.Commands.Add(command);
            var psObj = volPipeline.Invoke().FirstOrDefault();
            volPipeline = null;
            if (psObj == null)
            {
                var message = string.Format("Unable to find Volume {0}. Cannot continue with volume modification.", id);
                var errRecord = new ErrorRecord(new VolumeException(message), message, ErrorCategory.InvalidData, this);

                SolidFireUtilities.HandleError(errRecord, this, false);
                return null;
            }
            return (ElementVolume) psObj.BaseObject;
        }


        private void processVolumeChange(ElementVolume volBeforeModify)
        {
            var request = new ModifyVolumeRequest
            {
                VolumeID = volBeforeModify.VolumeID,
                AccountID = accountId,
                Access = !string.IsNullOrEmpty(Access) ? Access : null
            };


            if (ParameterSetName == "q")
            {
                if (minIOPS.HasValue || maxIOPS.HasValue || burstIOPS.HasValue || burstTime.HasValue)
                {
                    request.Qos = new QoS();
                    if (MinIOPS > -1)
                    {
                        request.Qos.MinIOPS = MinIOPS;
                    }
                    if (MaxIOPS > -1)
                    {
                        request.Qos.MaxIOPS = MaxIOPS;
                    }
                    if (BurstIOPS > -1)
                    {
                        request.Qos.BurstIOPS = BurstIOPS;
                    }
                }
            }
            else
            {
                if (associateWithQoSPolicy.HasValue && !qosPolicyID.HasValue && !volBeforeModify.QosPolicyID.HasValue)
                {
                    // no new QoSPolicyID or old QosPolicyID   
                    var message = "Missing QosPolicyID. Unable to Associate with QoS Policy because there is no QoSPolicyID to use. Please supply on.";
                    var errRecord = new ErrorRecord(new VolumeException(message), message, ErrorCategory.InvalidData, this);

                    SolidFireUtilities.HandleError(errRecord, this);
                }
                request.QosPolicyID = qosPolicyID;
                request.AssociateWithQoSPolicy = AssociateWithQoSPolicy;
            }
            
            long? newSize = totalSize;

            if (newSize.HasValue)
            {
                if (GB.IsPresent)
                {
                    newSize = newSize * Convert.ToInt64(Math.Pow(1000, 3));
                }
                if (GiB.IsPresent)
                {
                    newSize = newSize * Convert.ToInt64(Math.Pow(1024, 3));
                }
            }
            request.TotalSize = newSize;
            request.Attributes = Attributes;


            var objsFromAPI = SendRequest<ModifyVolumeResult>("ModifyVolume", request);

            foreach (var obj in objsFromAPI)
            {
                processedIDs.Add(volBeforeModify.VolumeID);
            }
        }

        protected override void EndProcessing()
        {
            base.EndProcessing();
            if (processedIDs.Count > 0)
            {
                var command = new Command("Get-SFVolume");
                command.Parameters.Add("VolumeID", processedIDs);
                if (SFConnection != null)
                {
                    command.Parameters.Add("SFConnection", SFConnection);
                }
                else if (Target != null)
                {
                    command.Parameters.Add("Target", Target);
                }
                pipeline.Commands.Add(command);
                WriteObject(pipeline.Invoke(), true);
                processedIDs.Clear();
            }
            pipeline = null;
        }

        #endregion
    }
}