﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core.Validation;
using System.Management.Automation.Runspaces;
using SolidFire.Core;
using SolidFire.Element.Api;
#endregion

namespace SolidFire.Volume.Set
{
    /// <summary>
    /// ModifyVolumePair is used to pause or restart replication between a pair of volumes.   
    /// </summary>
    
    [Cmdlet(VerbsCommon.Set, "SFVolumePair", SupportsShouldProcess = true, ConfirmImpact = ConfirmImpact.Medium)]
    public class SetSFVolumePair : SFCmdlet
    {
        #region Private Data

        private bool? _pausedManual;

        private Pipeline _pipeline;

        #endregion

        #region Parameters
        /// <summary>
        /// List of IDs of active paired volumes to be modified.
        /// </summary>
        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "List of IDs of active paired volumes to be modified.")]
        [ValidatePattern(SolidFireValidations.Numeric)]
        public long[] VolumeID { get; set; }

        /// <summary>
        /// Pause replication
        /// Valid values that can be entered:
        /// true: to pause volume replication.
        /// false: to restart volume replication.
        /// If no value is specified, no change in replication is performed.
        /// </summary>
        [Parameter(Position = 1, Mandatory = false, ValueFromPipeline = false, HelpMessage = "Pause replication.")]
        public bool PausedManual
        {
            get { return _pausedManual.HasValue ? _pausedManual.Value : false; }
            set { _pausedManual = value; }
        }

        /// <summary>
        /// Volume replication mode.
        /// Possible values:
        /// Async: Waits for system to acknowledge that data is stored on source before writing to the target.
        /// Sync: Does not wait for data transmission acknowledgment from source to begin writing data to the target.
        /// SnapshotsOnly: Only snapshots created on the source cluster 
        /// will be replicated. Active writes from the source volume will not be replicated.
        /// </summary>
        [Parameter(Position = 2, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "Volume replication mode.")]
        [ValidateSet("Async", "Sync", "SnapshotsOnly", IgnoreCase = false)]
        public string Mode { get; set; }

        #endregion

        #region Cmdlet Overrides
        
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            CheckConnection();
            _pipeline = Runspace.DefaultRunspace.CreateNestedPipeline();
        }

        protected override void ProcessRecord()
        {
            base.ProcessRecord();

            foreach (var id in VolumeID)
            {
                var request = new ModifyVolumePairRequest();
                request.VolumeID = id;
                request.PausedManual = _pausedManual;
                request.Mode = Mode;

                SendRequest<ModifyVolumePairResult>("ModifyVolumePair", request);
            }
        }

        protected override void EndProcessing()
        {
            base.EndProcessing();
            var command = new Command("Get-SFVolumePair");
            _pipeline.Commands.Add(command);
            // Send it to the pipline
            WriteObject(_pipeline.Invoke(), true);
            _pipeline = null;
        }
        #endregion
    }
}