﻿#region Copyright
/*
* Copyright © 2014-2016 NetApp, Inc. All Rights Reserved.
*
* CONFIDENTIALITY NOTICE: THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
* NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
* EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
*/
#endregion

#region Using Directives

using System.ComponentModel;
using System.Management.Automation;
using SolidFire.Core;
using SolidFire.Element.Api;
using System.Collections;
using System.Linq;
using SolidFire.Core.Objects;
using SolidFire.Exceptions;

#endregion

namespace SolidFire.Volume.Start
{
    /// <summary>
    /// Start-SFVolumeBackup is used to initialize a bulk volume read session on a specified volume. Only two bulk volume processes
    /// can run simultaneously on a volume.When the session is initialized, data is read from a SolidFire storage volume for the
    /// purposes of storing the data on an external backup source.The external data is accessed by a web server running on a
    /// SolidFire node. Communications and server interaction information for external data access is passed by a script running on
    /// the SolidFire storage system.
    /// 
    /// At the start of a bulk volume read operation, a snapshot of the volume is made and the snapshot is deleted when the read has
    /// completed. You can also read a snapshot of the volume by entering the ID of the snapshot as a parameter. Reading a
    /// previous snapshot does not create a new snapshot of the volume, nor does the previous snapshot be deleted when the read
    /// completes.
    /// </summary>
    
    [Cmdlet(VerbsLifecycle.Start, "SFVolumeBackup")]
    public class StartSFVolumeBackup : SFCmdlet, IDynamicParameters
    {
        #region Private Data

        // Prepare the DynamicParameter classes so they're ready when Frequency is selected.
        private BackupToS3DynamicParameters _contextBackupToS3 = new BackupToS3DynamicParameters();
        private BackupToSwiftDynamicParameters _contextBackupToSwift = new BackupToSwiftDynamicParameters();
        private BackupToSolidFireDynamicParameters _contextBackupToSolidFire = new BackupToSolidFireDynamicParameters();

        private long? _snapshotID;

        #endregion

        #region Parameters

        [Parameter(Position = 0, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The ID of the volume to be read.")]
        public long VolumeID { get; set; }

        [Parameter(Position = 1, Mandatory = true, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The format of the volume data. Can be: uncompressed or native.")]
        [ValidateSet("uncompressed", "native", IgnoreCase = false)]
        public string Format { get; set; }

        [Parameter(Position = 2, Mandatory = false, ValueFromPipeline = true, ValueFromPipelineByPropertyName = true, HelpMessage = "The ID of a previously created snapshot used for bulk volume reads.")]
        public long SnapshotID
        {
            get
            {
                return _snapshotID.HasValue ? _snapshotID.Value : 0;
            }
            set
            {
                _snapshotID = value;
            }
        }

        [Parameter(Position = 2, Mandatory = true, HelpMessage = "The target on which to store the backup of volume data. Can be: SolidFire, Swift, or S3.")]
        [ValidateSet("SolidFire", "Swift", "S3", IgnoreCase = false)]
        public string BackupTo { get; set; }


        #endregion

        #region Cmdlet Overrides
        protected override void BeginProcessing()
        {
            base.BeginProcessing();
            CheckConnection();
        }

        protected override void ProcessRecord()
        {
            base.ProcessRecord();
            var request = new StartBulkVolumeReadRequest()
            {
                VolumeID = VolumeID,
                Format = Format,
                SnapshotID = _snapshotID
            };
            var ht = new Hashtable();
            var rangeHt = new Hashtable();
            rangeHt.Add("lba", 0);
            rangeHt.Add("blocks", 244224);
            ht.Add("range", rangeHt);
            request.Script = "bv_internal.py";
            request.ScriptParameters = ht;

            switch (BackupTo)
            {
                case "S3":
                    _setS3Params(request);
                    break;
                case "Swift":
                    _setSwiftParams(request);
                    break;
                case "SolidFire":
                    _setSolidFireParams(request);
                    break;
            }

            var objsFromAPI = SendRequest<StartBulkVolumeReadResult>("StartBulkVolumeRead", request);

            WriteObject(objsFromAPI.Select(obj => obj.Result), true);
        }

        #endregion

        private void _setSwiftParams(StartBulkVolumeReadRequest request)
        {
            var writeHt = new Hashtable();
            writeHt.Add("url", _contextBackupToSwift.Url);
            writeHt.Add("username", _contextBackupToSwift.Username);
            writeHt.Add("authenticationKey", _contextBackupToSwift.AuthenticationKey);
            writeHt.Add("container", _contextBackupToSwift.Container);
            writeHt.Add("endpoint", "swift");
            writeHt.Add("format", Format);
            request.ScriptParameters.Add("write", writeHt);
        }

        private void _setS3Params(StartBulkVolumeReadRequest request)
        {
            var writeHt = new Hashtable();
            writeHt.Add("awsAccessKeyID", _contextBackupToS3.AccessKeyID);
            writeHt.Add("awsSecretAccessKey", _contextBackupToS3.SecretAccessKey);
            writeHt.Add("endpoint", "s3");
            writeHt.Add("format", Format);
            writeHt.Add("hostname", _contextBackupToS3.Hostname);
            writeHt.Add("bucket", _contextBackupToS3.Bucket);
            request.ScriptParameters.Add("write", writeHt);
        }

        private void _setSolidFireParams(StartBulkVolumeReadRequest request)
        {
            var writeHt = new Hashtable();
            writeHt.Add("mvip", _contextBackupToSolidFire.Mvip);
            writeHt.Add("username", _contextBackupToSolidFire.Username);
            writeHt.Add("endpoint", "solidfire");
            writeHt.Add("format", Format);
            writeHt.Add("key", _contextBackupToSolidFire.BulkVolumeWriteKey);
            request.ScriptParameters.Add("write", writeHt);
        }

        /// <summary>
        /// Override the BeforeRequest method in SFCmdlet which is called immediately before a request is made to a specific connection.
        /// This offers the opportunity to set a parameter in the request object based on the connection.
        /// </summary>
        /// <param name="request">The request object that can be modified and is about to be used in the HttpRequest. This is passed by ref.</param>
        /// <param name="method">The API method about to be called in the HttpRequest. This is passed by ref.</param>
        /// <param name="connection">The connection this request is about to be called on.</param>
        public override void BeforeRequest(ref RpcBase request, ref string method, SFConnection connection)
        {
            // Make sure we're working with the proper request object. This should never fail but if it does, throw a RequestException.
            if (request is StartBulkVolumeReadRequest)
            {
                var startBulkVolueReadRequest = (StartBulkVolumeReadRequest)request;
                // If restoring from S3 or Swift, we need to set the "prefix" value in the ScriptParameters Hashtable using information in the connection.
                if (BackupTo == "S3" || BackupTo == "Swift")
                {
                    // Get the volume by VolumeID or throw a RequestException
                    var activeRequest = new ListActiveVolumesRequest();
                    activeRequest.StartVolumeID = VolumeID;
                    activeRequest.Limit = 1;
                    var volume = SingleConnectionRequest<ListActiveVolumesResult>("ListActiveVolumes", connection, activeRequest);
                    if (volume.Result == null || volume.Result.Volumes.Count() != 1 || volume.Result.Volumes.First().VolumeID != VolumeID)
                    {
                        throw new RequestException(string.Format("Unable to find active volume for VolumeID: {0}", VolumeID));
                    }
                    var volumeName = volume.Result.Volumes.First().Name;

                    // Pull the "read" Hashtable from the ScriptParameters property in the request object or throw RequestException.
                    var readHt = (Hashtable)startBulkVolueReadRequest.ScriptParameters["write"];
                    if (readHt == null)
                    {
                        throw new RequestException("Improper mix of script parameters in 'backup to' request. Unable to find Hashtable 'write' in ScriptParameters Hashtable.");
                    }

                    var prefix = string.Format("{0}/{1}-{2}", connection.Name, volumeName, VolumeID);
                    if (BackupTo == "S3" && !string.IsNullOrEmpty(_contextBackupToS3.Nametag))
                    {
                        prefix = prefix + "/" + _contextBackupToS3.Nametag;
                    }
                    if (BackupTo == "Swift" && !string.IsNullOrEmpty(_contextBackupToSwift.Nametag))
                    {
                        prefix = prefix + "/" + _contextBackupToSwift.Nametag;
                    }

                    // Set the prefix value in the read Hashtable using the connection name, volume name, and volume ID
                    readHt.Add("prefix", prefix);
                }
            }
        }

        /// <summary>
        /// This is necessary to implement IDynamicParameters. It hands back the corresponding parameter set
        /// based on the value in the RetoreFrom parameter.
        /// </summary>
        /// <returns>DynamicParameter class with parameters for the selected RestoreFrom value</returns>
        public object GetDynamicParameters()
        {
            switch (BackupTo)
            {
                case "S3":
                    return _contextBackupToS3;
                case "Swift":
                    return _contextBackupToSwift;
                case "SolidFire":
                    return _contextBackupToSolidFire;
            }
            return null;
        }
    }
}
